/***************************************************************************
                          cconfmysql.cpp  -  description
                             -------------------
    begin                : Sun Jun 29 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan at verliba dot cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cconfmysql.h"
#include <algorithm>
using namespace std;

namespace nConfig
{



cConfMySQL::cConfMySQL( cMySQL &mysql) :
	mMySQL(mysql),mQuery(mMySQL)
{
}

cConfMySQL::~cConfMySQL()
{
}

int cConfMySQL::Save()
{
	SavePK(false);
	return 0;
}

int cConfMySQL::Load()
{
	MYSQL_ROW row;
	if(!(row= mQuery.Row())) return -1;

	int i = 0;
	for_each(mhItems.begin(), mhItems.end(), ufLoad(row));
	return 0;
}

/** return -1 on error, otherwinse number of results */
int cConfMySQL::StartQuery()
{
	int ret = mQuery.Query();
	if(ret <= 0)
	{
		mQuery.Clear();
		return ret;
	}
	mQuery.StoreResult();
	mCols = mQuery.Cols();
	return ret;
}

/** return -1 on error, otherwinse number of results */
int cConfMySQL::StartQuery(string query)
{
	mQuery.OStream() << query;
	return StartQuery();
}

int cConfMySQL::EndQuery()
{
	mQuery.Clear();
	return 0;
}

};


/*!
    \fn nConfig::cConfMySQL::AddPrimaryKey(const char*)
 */
void nConfig::cConfMySQL::AddPrimaryKey(const char*key)
{
	string Key(key);
	size_t Hash = msHasher(Key);
	cConfigItemBase *item = mhItems.GetByHash(Hash);
	if( item != NULL) mPrimaryKey.AddWithHash(item,Hash);
}


/*!
    \fn nConfig::cConfMySQL::WherePKey(ostream &os)
 */
void nConfig::cConfMySQL::WherePKey(ostream &os)
{
	os << " WHERE (";
	AllPKFields(os , true, true, false, string(" AND "));
	os << " )";
}


/*!
    \fn nConfig::cConfMySQL::AllFields(ostream &, bool DoF=true, bool DoV=false, string joint)
 */
void nConfig::cConfMySQL::AllFields(ostream &os, bool DoF, bool DoV, bool IsAff, string joint)
{
	 for_each(mhItems.begin(), mhItems.end(), ufEqual(os, joint , DoF, DoV, IsAff));
}

/*!
    \fn nConfig::cConfMySQL::AllPKFields(ostream &, bool DoF=true, bool DoV=false, string joint)
 */
void nConfig::cConfMySQL::AllPKFields(ostream &os, bool DoF, bool DoV, bool IsAff, string joint)
{
	 for_each(mPrimaryKey.begin(), mPrimaryKey.end(), ufEqual(os, joint , DoF, DoV, IsAff));
}

/*!
    \fn nConfig::cConfMySQL::SelectFields(ostream &)
 */
void nConfig::cConfMySQL::SelectFields(ostream &os)
{
	os << "SELECT ";
	AllFields(os, true, false, false,string(", "));
	os << " FROM " << mTableName << " ";
}


/*!
    \fn nConfig::cConfMySQL::UpdateFields(ostream &os)
 */
void nConfig::cConfMySQL::UpdateFields(ostream &os)
{
	os << "UPDATE " << mTableName  << " SET ";
	AllFields(mQuery.OStream(), true, true, true, string(", "));
}


/*!
    \fn nConfig::cConfMySQL::LoadPK()
 */
bool nConfig::cConfMySQL::LoadPK()
{
	ostringstream query;

	SelectFields( query );
	WherePKey( query );

	if( StartQuery(query.str()) == -1) return false;
	bool found = (Load() >= 0);
	EndQuery();
	return found;
}


/*!
    \fn nConfig::cConfMySQL::SavePK(bool dup)
 */
bool nConfig::cConfMySQL::SavePK(bool dup)
{
	mQuery.OStream() << "INSERT IGNORE INTO " << mTableName << " (";
	AllFields(mQuery.OStream(), true, false, false, string(", "));
	mQuery.OStream() << ") VALUES (";
	AllFields(mQuery.OStream(), false, true, true, string(", "));
	mQuery.OStream() << ")";
	if(dup)
	{
		mQuery.OStream() << " ON DUPLICATE SET ";
		AllFields(mQuery.OStream(), true, true, true, string(", "));
	}
	bool ret = mQuery.Query();
	mQuery.Clear();
	return ret;
}


/*!
    \fn nConfig::cConfMySQL::DeletePK()
 */
void nConfig::cConfMySQL::DeletePK()
{
	mQuery.Clear();
	mQuery.OStream() << "DELETE FROM " << mTableName << " ";
	WherePKey( mQuery.OStream() );
	mQuery.Query();
	mQuery.Clear();
}

nConfig::cConfMySQL::db_iterator &nConfig::cConfMySQL::db_begin()
{
	if((-1 == this->StartQuery()) || (Load() < 0))
		mDBBegin = NULL;
	else
		mDBBegin = this;
	return mDBBegin;
}

nConfig::cConfMySQL::db_iterator &nConfig::cConfMySQL::db_iterator::operator++()
{
	if (mConf != NULL)
	{
		if (mConf->Load() < 0)
		{
			mConf->EndQuery();
			mConf = NULL;
		}
	}
	return *this;
}

#ifndef _WIN32
std::ostream &nConfig::cConfigItemMySQL<std::string>::WriteToStream (std::ostream& os)
{
	if( (this->operator std::string()).size() )
	{
		os << '"';
		cConfigItemBaseT<std::string>::WriteToStream(os);
		os << '"';
	} else os << " NULL ";

	return os;
}
#else
std::ostream &nConfig::cConfigItemMySQL<std::string>::WriteToStream (std::ostream& os)
{
	if( (this->AsString()).size() )
	{
		os << '"';
		cConfigItemBaseT<std::string>::WriteToStream(os);
		os << '"';
	} else os << " NULL ";

	return os;
}
#endif


/*!
    \fn nConfig::cConfMySQL::UpdatePKVar(const char* var_name, string &new_val)
 */
bool nConfig::cConfMySQL::UpdatePKVar(const char* var_name, string &new_val)
{
	cConfigItemBase * item = NULL;
	string var(var_name);
	item = operator[](var);
	if(!item) return false;
	LoadPK();
	item->ConvertFrom(new_val);
	return UpdatePKVar(item);
}


/*!
    \fn nConfig::cConfMySQL::UpdatePKVar(cConfigItemBase *item)
 */
bool nConfig::cConfMySQL::UpdatePKVar(cConfigItemBase *item)
{
	mQuery.OStream() << "UPDATE " << mTableName << " SET ";
	ufEqual(mQuery.OStream(), string(", "), true, true, true)(item);
	WherePKey(mQuery.OStream());
	bool ret = mQuery.Query();
	mQuery.Clear();
	return ret;
}


/*!
    \fn nConfig::cConfMySQL::UpdatePK()
 */
bool nConfig::cConfMySQL::UpdatePK()
{
	UpdateFields(mQuery.OStream());
	WherePKey(mQuery.OStream());
	bool ret = mQuery.Query();
	mQuery.Clear();
	return ret;
}


/*!
    \fn nConfig::cConfMySQL::UpdatePKVar(const char *field)
 */
bool nConfig::cConfMySQL::UpdatePKVar(const char *field)
{
	cConfigItemBase *item = operator[](field);
	if(!item) return false;
	return UpdatePKVar(item);
}

void nConfig::cConfMySQL::WriteStringConstant(ostream &os, const string &str)
{
	string tmp;

	// replace all \ by \\
	// replace all " by \"
	int pos = 0, lastpos = 0;
	char c;

	while (
		str.npos != lastpos &&
		str.npos != ( pos = str.find_first_of("\\\"'`",lastpos)))
	{
		tmp.append(str, lastpos , pos - lastpos );
		tmp.append("\\");
		c = str[pos];
		tmp.append(&c,1);
		lastpos = pos + 1;
	}

	if( str.npos != lastpos) tmp.append(str, lastpos , pos - lastpos );
	os << tmp;
}

void nConfig::cConfMySQL::ufEqual::operator()(cConfigItemBase* item)
{
	if(!start) mOS << mJoint; else start=false;
	if(mDoField) mOS << (item->mName);
	if(mDoValue) {
		bool IsStr = false;
		bool IsNum = false;
		bool IsNull = false;
#ifndef _WIN32
		if(!IsStr && dynamic_cast<cConfigItemBaseT<std::string>*>(item)){
			IsStr = true;
			IsNull = !(dynamic_cast<cConfigItemBaseT<std::string>*>
				(item)->operator std::string()).size();
		}
		if(!IsStr && dynamic_cast<cConfigItemBaseT<char *     >*>(item))
		{
			IsStr = true;
			IsNull = !strlen(dynamic_cast<cConfigItemBaseT<char *>*>
				(item)->operator char*());
		}
		if(!IsStr && dynamic_cast<cConfigItemBaseT<time_t>*>(item))
		{
			IsNum = true;
			IsNull = !(dynamic_cast<cConfigItemBaseT<time_t>*>(item)->operator time_t());
		}
#else
		if(!IsStr && dynamic_cast<cConfigItemBaseT<std::string>*>(item)){
			IsStr = true;
			IsNull = !(dynamic_cast<cConfigItemBaseT<std::string>*>
				(item)->AsString()).size();
		}
		if(!IsStr && dynamic_cast<cConfigItemBaseT<char *     >*>(item))
		{
			IsStr = true;
			IsNull = !strlen(dynamic_cast<cConfigItemBaseT<char *>*>
				(item)->AsPChar());
		}
		if(!IsStr && dynamic_cast<cConfigItemBaseT<time_t>*>(item))
		{
			IsNum = true;
			IsNull = !(dynamic_cast<cConfigItemBaseT<time_t>*>(item)->AsUnsigned());
		}
#endif

		if(mDoField)
		{
			if(IsNull && !mIsAffect) mOS << " IS ";
			else mOS << " = ";
		}

		if(IsStr && !IsNull) mOS << '"';
		if(IsNull) mOS << "NULL ";
		else
		{
			if(!IsStr) item->WriteToStream(mOS);
			else
			{
				string tmp;
				item->ConvertTo(tmp);
				cConfMySQL::WriteStringConstant(mOS, tmp);
			}
		}
		if(IsStr && !IsNull) mOS << '"';
	}
}


