/***************************************************************************
                          creglist.cpp  -  description
                             -------------------
    begin                : Mon Jul 7 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan at verliba dot cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cserverdc.h"
#include "creglist.h"
#include "cuser.h"
#include "creguserinfo.h"
#include <sstream>
#include <cctype>
#include <algorithm>
#include <unistd.h>

namespace nDirectConnect
{
namespace nTables
{


cRegList::cRegList(cMySQL &mysql): cConfMySQL(mysql)
	,mCache(mysql,"reglist", "nick", "reg_date")
{
	cTime now;
	SetClassName("nDC::cRegList");
	mTableName="reglist";

	Add("nick",mModel.mNick);
	AddPrimaryKey("nick");
	Add("class", mModel.mClass);
	Add("class_protect", mModel.mClassProtect);
	Add("class_hidekick", mModel.mClassHideKick);
	Add("hide_kick", mModel.mHideKick);
	Add("reg_date",mModel.mRegDate);
	Add("reg_op",mModel.mRegOp);
	Add("pwd_change", mModel.mPwdChange);
	Add("pwd_crypt", mModel.mPWCrypt);
	Add("login_pwd",mModel.mPasswd);
	Add("login_last", mModel.mLoginLast);
	Add("logout_last", mModel.mLogoutLast);
	Add("login_cnt",mModel.mLoginCount);
	Add("login_ip", mModel.mLoginIP);
	Add("error_last", mModel.mErrorLast);
	Add("error_cnt", mModel.mErrorCount);
	Add("error_ip", mModel.mErrorIP);
	Add("enabled", mModel.mEnabled);
	Add("email", mModel.mEmail);
	Add("note_op", mModel.mNoteOp);
	Add("note_usr", mModel.mNoteUsr);
	SetBaseTo(&mModel);
}

cRegList::~cRegList(){
}

/** creates empty table in database if not exists */
void cRegList::CreateTable()
{
	mQuery.OStream() <<
		"CREATE TABLE IF NOT EXISTS reglist ("
		"nick varchar(30) NOT NULL PRIMARY KEY, " // users's nick
		"class int(2) default 1, " // user's class
		"class_protect int(2) default 0, " //  user's protection class
		"class_hidekick int(2) default 0, " // user's hidekick class
		"hide_kick bool default 0, "
		"reg_date int(11), " // registration date
		"reg_op varchar(30), " // registration operator, the one who gave the registration
		"pwd_change bool default 1, " // user must change password
		"pwd_crypt bool default 0, " // crypt password??
		"login_pwd varchar(60), " // password crypted or not
		"login_last int(11) default 0, " // last login
		"logout_last int(11) default 0, " // last login
		"login_cnt int(11) default 0, " // succesfull login count
		"login_ip varchar(16), " // last ip
		"error_last int(11), " // unsuccessfull last login date
		"error_cnt int(11) default 0, " // unsuccessfull login count
		"error_ip varchar(16), " // ip on unsuccessfull login
		"enabled bool default 1, " // account enabled
		"email varchar(60), " // user's email
		"note_op text, "
		"note_usr text, "
		"INDEX login_index (login_last), "
		"INDEX logout_index (logout_last) "
		")";

	mQuery.Query();
	mQuery.Clear();
}

/** find nick in reglist
if not foud return 0
else return 1 and fill in the reuserinfo parameter */
bool cRegList::FindRegInfo(cRegUserInfo &ui, const string &nick)
{
	if(mCache.IsLoaded() && !mCache.Find(nick)) return false;//@todo nick2dbkey
	SetBaseTo(&ui);
	ui.mNick = nick;//@todo nick2dbkey
	return LoadPK();
}

/** add registered user */
bool cRegList::AddRegUser(const string &nick, cConnDC *op, int cl, const char *password)
{
	cRegUserInfo ui;

	if(FindRegInfo(ui, nick)) return false;
	ui.mNick = nick;//@todo nick2dbkey
	ui.mClass = cl;
	ui.mRegDate = cTime().Sec();
	ui.mRegOp   = (op && op->mpUser)?op->mpUser->mNick:string("hub-security");

	if(password) ui.SetPass(string(password));
	else ui.SetPass(string());

	if(mCache.IsLoaded()) mCache.Add(nick);//@todo nick2dbkey

	SetBaseTo(&ui);
	return SavePK();
}


/** No descriptions */
bool cRegList::ChangePwd(const string &nick, const string &pwd, int crypt)
{
	if(!FindRegInfo(mModel, nick)) return false;
	mModel.SetPass(pwd);
	return UpdatePK();
}

/** No descriptions */
bool cRegList::SetVar(const string &nick, string &field, string &value)
{
	SetBaseTo(&mModel);
	mModel.mNick = nick; //@todo nick2dbkey
	return UpdatePKVar(field.data(), value);
}

/** log that user logged in */
bool cRegList::Login(cConnDC *conn, const string &nick)
{
	cRegUserInfo ui;
	if(!FindRegInfo(ui, nick)) return false;
	ui.mLoginLast = cTime().Sec();
	ui.mLoginIP   = conn->AddrIP();
	ui.mLoginCount ++;
	return UpdatePK();
}

/** log that user logged in */
bool cRegList::Logout(const string &nick)
{
	if(!FindRegInfo(mModel, nick)) return false;
	mModel.mLogoutLast = cTime().Sec()-1; // this is a patch for users that connect twice
	return UpdatePKVar("logout_last");
}

/** log that user logged in with error*/
bool cRegList::LoginError(cConnDC *conn, const string &nick)
{
	if(!FindRegInfo(mModel, nick)) return false;
	mModel.mErrorLast = cTime().Sec();
	mModel.mErrorIP = conn->AddrIP();
	return UpdatePK();
}


/*!
    \fn nDirectConnect::nTables::cRegList::DelReg(const string &nick)
 */
bool cRegList::DelReg(const string &nick)
{
	if(!FindRegInfo(mModel, nick)) return false;
	DeletePK();
	return true;
}

/**

How registering works:
----------------------

1/ user <Nick> asks to be registered
2/ if operator is ok with it, he uses command !regnewuser <Nick> [<class>=1] to prepare an account for the user..
	* this checks if such user is registered (if so error occurs)
	* creates default reguser data into the reglist, which is marked as pwd_change
	* then it sends pm to the user asking him por passwrd and some other settings
3/ user provides password with +passwd <new_pass> [<crypt>=0], which is enabled only if pwd_change is marked
	* if user doesn't provide any password he won't be registered
	* if password is provided and is ok, user finishes his registration
4/ to change password user asks any op again, op uses command !regpasswd <Nick> to enable password changing
5/ to change user's class op uses commands !regclass <Nick> <new_class> ; new_class must be less then op's class
6/ to ungegiser use op command !regdeluser <Nick> - this disables the registration by setting a flag
7/ to purge a disabled record, admin can do !regpurge <Nick>

Note: registered users aren't banned by ip only, but by their nick too

1. method
---------
crypted=crypt("heslo"); // mysql: encrypt; php, perl, C : crypt
if(crypt("heslo2",crypted) == crypted) OK

md5
crypt
*/

};
};


